@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2021 Xorg
+*    Copyright © 2014-2022 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -68,6 +68,12 @@
 # include <GLFW/glfw3.h>
 #endif
 
+#if HAS_OpenCL
+# define CL_TARGET_OPENCL_VERSION 120
+# include <CL/cl.h>
+# include "opencl_ext.h"
+#endif
+
 #if HAS_LIBSTATGRAB
 # include <statgrab.h>
 #endif
@@ -232,6 +238,9 @@ static int cpu_technology(Labels *data)
 			casprintf(&data->tab_cpu[VALUE][TECHNOLOGY], false, "%i nm", db[i].process);
 			RETURN_OR_EXIT(0);
 		}
+		else
+			MSG_DEBUG("cpu_technology: model: %i/%i, ext. model: %i/%i, ext. family: %i/%i",
+			          db[i].cpu_model, l_data->cpu_model, db[i].cpu_ext_model, l_data->cpu_ext_model, db[i].cpu_ext_family, l_data->cpu_ext_family);
 	}
 
 	MSG_WARNING(_("Your CPU is not present in the database ==> %s, model: %i, ext. model: %i, ext. family: %i"),
@@ -244,7 +253,8 @@ static int cpu_technology(Labels *data)
 static int call_libcpuid_static(Labels *data)
 {
 	int err, i, j = 0;
-	char tmp[MAXSTR * 2] = "";
+	bool cpu_supported       = true;
+	char tmp[MAXSTR * 2]     = "";
 	const char *fmt_cpuid    = opts->cpuid_decimal ? "%i" : "0x%X";
 	const char *fmt_cache_kb = _("%d x %d %s, %d-way");
 	const char *fmt_cache_mb = _("%d %s, %d-way");
@@ -269,6 +279,23 @@ static int call_libcpuid_static(Labels *data)
 		return 1;
 	}
 
+	/* Unsupported CPUs
+	   When CPUID.07H.0H:EDX[15]=1, hybrid topology is present
+	   https://www.intel.com/content/www/us/en/developer/articles/guide/alder-lake-developer-guide.html?wapkw=alder%20lake
+	*/
+	if((datanr.vendor == VENDOR_INTEL) && (raw.basic_cpuid[0x7][EDX] & 0x8000))
+	{
+		MSG_WARNING(_("CPU hybrid architecture is not supported. For more details, please refer to following issue: %s"), "https://github.com/anrieff/libcpuid/issues/157");
+		cpu_supported               = false;
+		datanr.num_cores            = 0;
+		datanr.num_logical_cpus     = 0;
+		datanr.l1_data_cache        = 0;
+		datanr.l1_instruction_cache = 0;
+		datanr.l2_cache             = 0;
+		datanr.l3_cache             = 0;
+		datanr.l4_cache             = 0;
+	}
+
 	/* Some prerequisites */
 	data->cpu_count              = datanr.num_logical_cpus;
 	data->l_data->cpu_model      = datanr.model;
@@ -278,7 +305,7 @@ static int call_libcpuid_static(Labels *data)
 		opts->selected_core = 0;
 
 	/* Basically fill CPU tab */
-	casprintf(&data->tab_cpu[VALUE][CODENAME],      false, "%s",      datanr.cpu_codename);
+	casprintf(&data->tab_cpu[VALUE][CODENAME],      false, "%s",      cpu_supported ? datanr.cpu_codename : _("NOT SUPPORTED"));
 	casprintf(&data->tab_cpu[VALUE][SPECIFICATION], false, "%s",      datanr.brand_str);
 	casprintf(&data->tab_cpu[VALUE][FAMILY],        false, fmt_cpuid, datanr.family);
 	casprintf(&data->tab_cpu[VALUE][EXTFAMILY],     false, fmt_cpuid, datanr.ext_family);
@@ -464,7 +491,7 @@ static int call_libcpuid_msr_dynamic(Labels *data)
 
 	MSG_VERBOSE("%s", _("Calling libcpuid for retrieving CPU MSR dynamic values"));
 	SEND_DATA(&data->socket_fd,  &cmd, sizeof(DaemonCommand));
-	SEND_DATA(&data->socket_fd,  &opts->selected_core, sizeof(unsigned));
+	SEND_DATA(&data->socket_fd,  &opts->selected_core, sizeof(uint8_t));
 	RECEIVE_DATA(&data->socket_fd, &msg, sizeof(MsrDynamicData));
 
 	/* CPU Voltage */
@@ -802,6 +829,179 @@ clean:
 	return err;
 }
 
+#define CLINFO(dev_id, PARAM, prop) \
+	clGetDeviceInfo(dev_id, PARAM, sizeof(prop), &prop, NULL)
+static int get_gpu_comp_unit (struct pci_dev *dev, uint32_t *comp_unit, char *comp_unit_type)
+{
+	int ret_cl = 0;
+#if HAS_OpenCL
+	uint8_t ret_domain_nv = 0, ret_bus_nv = 0, ret_dev_nv = 0;
+	char platform_name[MAXSTR] = "", platform_version[MAXSTR] = "", device_name[MAXSTR] = "", device_version[MAXSTR] = "";
+	cl_uint num_pf = 0, num_ocl_dev = 0, ocl_vendor = 0, amd_gfx_major = 0;
+	cl_uint i, j;
+	cl_platform_id *platforms = NULL;
+	cl_device_id *devices = NULL;
+	cl_device_topology_amd topo_amd; // for AMD
+	cl_uint ocl_domain_nv, ocl_bus_nv, ocl_dev_nv; // for NVIDIA
+
+	ret_cl = clGetPlatformIDs(0, NULL, &num_pf); // get number of platform
+	if (ret_cl != CL_SUCCESS || num_pf == 0)
+	{
+		MSG_WARNING(_("There is no platform with OpenCL support (%s)"), opencl_error(ret_cl));
+		return ret_cl;
+	}
+	MSG_DEBUG("Number of OpenCL platforms: %u", num_pf);
+
+	platforms = (cl_platform_id*) malloc(sizeof(cl_platform_id) * num_pf);
+	ret_cl = clGetPlatformIDs(num_pf, platforms, NULL);; // get all platforms
+	if (ret_cl != CL_SUCCESS)
+	{
+		MSG_ERROR(_("failed to get all OpenCL platforms (%s)"), opencl_error(ret_cl));
+		free(platforms);
+		return ret_cl;
+	}
+
+	for (i = 0; i < num_pf; i++) // find GPU devices
+	{
+		ret_cl = clGetPlatformInfo(platforms[i], CL_PLATFORM_NAME, sizeof(platform_name), platform_name, NULL); // get platform name
+		if (ret_cl != CL_SUCCESS)
+		{
+			MSG_ERROR(_("failed to get name for platform %u (%s)"), i, opencl_error(ret_cl));
+			continue;
+		}
+
+		ret_cl = clGetPlatformInfo(platforms[i], CL_PLATFORM_VERSION, sizeof(platform_version), platform_version, NULL); // get platform version
+		if (ret_cl != CL_SUCCESS)
+		{
+			MSG_ERROR(_("failed to get version for platform %u (%s)"), i, opencl_error(ret_cl));
+			continue;
+		}
+
+		ret_cl = clGetDeviceIDs(platforms[i], CL_DEVICE_TYPE_ALL, 0, NULL, &num_ocl_dev); // get number of device
+		if (ret_cl != CL_SUCCESS)
+		{
+			MSG_ERROR(_("failed to find number of OpenCL devices for platform '%s %s' (%s)"), platform_name, platform_version, opencl_error(ret_cl));
+			continue;
+		}
+
+		devices = (cl_device_id*) malloc(sizeof(cl_device_id) * num_ocl_dev);
+		ret_cl = clGetDeviceIDs(platforms[i], CL_DEVICE_TYPE_ALL, num_ocl_dev, devices, NULL); // get all devices
+		if (ret_cl != CL_SUCCESS)
+		{
+			MSG_ERROR(_("failed to get all of OpenCL devices for platform '%s %s' (%s)"), platform_name, platform_version, opencl_error(ret_cl));
+			continue;
+		}
+		MSG_DEBUG("Number of OpenCL devices for platform '%s %s': %u", platform_name, platform_version, num_ocl_dev);
+
+		for (j = 0; j < num_ocl_dev; j++)
+		{
+			CLINFO(devices[j], CL_DEVICE_VENDOR_ID, ocl_vendor);
+			if (dev->vendor_id != ocl_vendor)
+				continue;
+
+			ret_cl = clGetDeviceInfo(devices[j], CL_DEVICE_NAME, sizeof(device_name), device_name, NULL);
+			if (ret_cl != CL_SUCCESS)
+			{
+				MSG_ERROR(_("failed to get name for device %u (%s)"), j, opencl_error(ret_cl));
+				continue;
+			}
+
+			ret_cl = clGetDeviceInfo(devices[j], CL_DEVICE_VERSION, sizeof(device_version), device_version, NULL);
+			if (ret_cl != CL_SUCCESS)
+			{
+				MSG_ERROR(_("failed to get version for device %u (%s)"), j, opencl_error(ret_cl));
+				continue;
+			}
+
+			MSG_DEBUG("OpenCL device %u is '%s %s'", j, device_name, device_version);
+			switch (ocl_vendor)
+			{
+				case DEV_VENDOR_ID_AMD:
+					ret_cl = CLINFO(devices[j], CL_DEVICE_TOPOLOGY_AMD, topo_amd);
+					if (ret_cl != CL_SUCCESS)
+					{
+						MSG_WARNING(_("OpenCL driver for '%s %s' does not support CL_DEVICE_TOPOLOGY_AMD (%s)"), device_name, device_version, opencl_error(ret_cl));
+						continue;
+					}
+
+					if (dev->bus  ==  topo_amd.pcie.bus     &&
+					    dev->dev  ==  topo_amd.pcie.device  &&
+					    dev->func ==  topo_amd.pcie.function)
+					{
+						ret_cl = CLINFO(devices[j], CL_DEVICE_GFXIP_MAJOR_AMD, amd_gfx_major);
+						if (ret_cl != CL_SUCCESS)
+						{
+							MSG_WARNING(_("OpenCL driver for '%s %s' does not support CL_DEVICE_GFXIP_MAJOR_AMD (%s)"), device_name, device_version, opencl_error(ret_cl));
+							amd_gfx_major = 0;
+						}
+
+						if (10 <= amd_gfx_major) // for RDNA, gfx10+
+							snprintf(comp_unit_type, MAXSTR, "%s", "WGP"); // Workgroup Processor
+						else
+							snprintf(comp_unit_type, MAXSTR, "%s", "CU"); // Compute Unit
+
+						ret_cl = CLINFO(devices[j], CL_DEVICE_MAX_COMPUTE_UNITS, *comp_unit);
+						if (ret_cl != CL_SUCCESS)
+						{
+							MSG_ERROR(_("OpenCL driver for '%s %s' does not support CL_DEVICE_MAX_COMPUTE_UNITS (%s)"), device_name, device_version, opencl_error(ret_cl));
+							*comp_unit = 0;
+							continue;
+						}
+					}
+					break;
+				case DEV_VENDOR_ID_INTEL:
+					ret_cl = CLINFO(devices[j], CL_DEVICE_MAX_COMPUTE_UNITS, *comp_unit);
+					if (ret_cl != CL_SUCCESS)
+					{
+						MSG_ERROR(_("OpenCL driver for '%s %s' does not support CL_DEVICE_MAX_COMPUTE_UNITS (%s)"), device_name, device_version, opencl_error(ret_cl));
+						*comp_unit = 0;
+						continue;
+					}
+					snprintf(comp_unit_type, MAXSTR, "%s", "EU"); // Execution Unit
+					break;
+				case DEV_VENDOR_ID_NVIDIA:
+					ret_domain_nv = CLINFO(devices[j], CL_DEVICE_PCI_DOMAIN_ID_NV, ocl_domain_nv);
+					ret_bus_nv    = CLINFO(devices[j], CL_DEVICE_PCI_BUS_ID_NV,    ocl_bus_nv);
+					ret_dev_nv    = CLINFO(devices[j], CL_DEVICE_PCI_SLOT_ID_NV,   ocl_dev_nv); // Slot == Device
+
+					if ((ret_domain_nv != CL_SUCCESS) || (ret_bus_nv != CL_SUCCESS) || (ret_dev_nv != CL_SUCCESS))
+					{
+						MSG_WARNING(_("OpenCL driver for '%s %s' does not support CL_DEVICE_PCI_DOMAIN_ID_NV (%s), CL_DEVICE_PCI_BUS_ID_NV (%s) or CL_DEVICE_PCI_SLOT_ID_NV (%s)"),
+						            device_name, device_version, opencl_error(ret_domain_nv), opencl_error(ret_bus_nv), opencl_error(ret_dev_nv));
+						continue;
+					}
+
+					if (dev->domain == (int) ocl_domain_nv &&
+					    dev->bus    ==       ocl_bus_nv    &&
+					    dev->dev    ==       ocl_dev_nv)
+					{
+						ret_cl = CLINFO(devices[j], CL_DEVICE_MAX_COMPUTE_UNITS, *comp_unit);
+						if (ret_cl != CL_SUCCESS)
+						{
+							MSG_ERROR(_("OpenCL driver for '%s %s' does not support CL_DEVICE_MAX_COMPUTE_UNITS (%s)"), device_name, device_version, opencl_error(ret_cl));
+							*comp_unit = 0;
+							continue;
+						}
+						snprintf(comp_unit_type, MAXSTR, "%s", "SM"); // Streaming Multiprocessor
+					}
+					break;
+				default:
+					MSG_WARNING(_("OpenCL is not supported with your GPU vendor (0x%X)"), ocl_vendor);
+					break;
+			} /* end switch (vendor_id) */
+		} /* end num_ocl_dev */
+		free(devices);
+	} /* end num_pf */
+	free(platforms);
+#else
+	UNUSED(dev);
+	UNUSED(comp_unit);
+	UNUSED(comp_unit_type);
+#endif /* HAS_OpenCL */
+
+	return ret_cl;
+}
+
 #define DEVICE_VENDOR_STR(d)  pci_lookup_name(pacc, buff, MAXSTR, PCI_LOOKUP_VENDOR, d->vendor_id, d->device_id)
 #define DEVICE_PRODUCT_STR(d) pci_lookup_name(pacc, buff, MAXSTR, PCI_LOOKUP_DEVICE, d->vendor_id, d->device_id)
 /* Find some PCI devices, like chipset and GPU */
@@ -810,9 +1010,10 @@ static int find_devices(Labels *data)
 	/* Adapted from http://git.kernel.org/cgit/utils/pciutils/pciutils.git/tree/example.c */
 	bool chipset_found = false;
 	char *gpu_vendor = NULL;
-	char gpu_driver[MAXSTR] = "", gpu_umd[MAXSTR] = "", buff[MAXSTR] = "";
+	char gpu_driver[MAXSTR] = "", gpu_umd[MAXSTR] = "", buff[MAXSTR] = "", comp_unit_type[MAXSTR] = "";
 	struct pci_access *pacc;
 	struct pci_dev *dev;
+	uint32_t comp_unit = 0;
 
 	MSG_VERBOSE("%s", _("Finding devices"));
 	pacc = pci_alloc(); /* Get the pci_access structure */
@@ -851,9 +1052,18 @@ static int find_devices(Labels *data)
 		{
 			switch(dev->vendor_id)
 			{
-				case 0x1002: gpu_vendor = "AMD";    break;
-				case 0x8086: gpu_vendor = "Intel";  break;
-				case 0x10DE: gpu_vendor = "NVIDIA"; break;
+				case DEV_VENDOR_ID_AMD:
+					gpu_vendor = "AMD";
+					data->g_data->bar_size[data->gpu_count] = dev->size[0];
+					break;
+				case DEV_VENDOR_ID_INTEL:
+					gpu_vendor = "Intel";
+					data->g_data->bar_size[data->gpu_count] = 0;
+					break;
+				case DEV_VENDOR_ID_NVIDIA:
+					gpu_vendor = "NVIDIA";
+					data->g_data->bar_size[data->gpu_count] = dev->size[1];
+					break;
 				default:     gpu_vendor = DEVICE_VENDOR_STR(dev);
 				             MSG_WARNING(_("Your GPU vendor is unknown: %s (0x%X)"), gpu_vendor, dev->vendor_id);
 			}
@@ -863,11 +1073,14 @@ static int find_devices(Labels *data)
 			find_gpu_device_path(dev, &data->g_data->device_path[data->gpu_count]);
 			find_gpu_kernel_driver(data->g_data->device_path[data->gpu_count], gpu_driver, &data->g_data->gpu_driver[data->gpu_count]);
 			find_gpu_user_mode_driver(data->g_data->gpu_driver[data->gpu_count], gpu_umd);
+			get_gpu_comp_unit(dev, &comp_unit, comp_unit_type);
+
 			casprintf(&data->tab_graphics[VALUE][GPU1VENDOR + data->gpu_count * GPUFIELDS], false, "%s", gpu_vendor);
 			casprintf(&data->tab_graphics[VALUE][GPU1DRIVER + data->gpu_count * GPUFIELDS], false, "%s", gpu_driver);
 			casprintf(&data->tab_graphics[VALUE][GPU1UMD    + data->gpu_count * GPUFIELDS], false, "%s", gpu_umd);
 			casprintf(&data->tab_graphics[VALUE][GPU1MODEL  + data->gpu_count * GPUFIELDS], false, "%s", DEVICE_PRODUCT_STR(dev));
 			casprintf(&data->tab_graphics[VALUE][GPU1DIDRID + data->gpu_count * GPUFIELDS], false, "0x%04X:0x%02X", dev->device_id, pci_read_byte(dev, PCI_REVISION_ID));
+			casprintf(&data->tab_graphics[VALUE][GPU1CU     + data->gpu_count * GPUFIELDS], true,  "%d %s", comp_unit, comp_unit_type);
 			data->gpu_count++;
 		}
 	}
@@ -1165,6 +1378,9 @@ skip_clocks:
 			casprintf(&data->tab_graphics[VALUE][GPU1PCIE        + i * GPUFIELDS], false,
 				_("Current: PCIe Gen%1dx%d / Max: Gen%1dx%d"),
 				pcie_sta_gen, atoi(pcie_sta_width), pcie_max_gen, atoi(pcie_max_width));
+		if(strlen(data->tab_graphics[VALUE][GPU1REBAR + i * GPUFIELDS]) == 0 && !ret_vram_total)
+			casprintf(&data->tab_graphics[VALUE][GPU1REBAR       + i * GPUFIELDS], false,
+				"%s", (atol(vram_total) / divisor_vram * 0.9 < data->g_data->bar_size[i] / (1 << 20)) ? _("Enabled") : _("Disabled"));
 
 		if(ret_temp && ret_load && ret_gclk && ret_mclk && ret_vram_used && ret_vram_total && ret_gvolt && ret_gpwr
 		&& ret_pcie_max_speed && ret_pcie_max_width && ret_pcie_sta_speed && ret_pcie_sta_width)
