@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2021 Xorg
+*    Copyright © 2014-2022 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -64,9 +64,9 @@ static void labels_setname(Labels *data)
 	asprintf(&data->tab_cpu[NAME][VOLTAGE],         _("Voltage"));
 	asprintf(&data->tab_cpu[NAME][SPECIFICATION],   _("Specification"));
 	asprintf(&data->tab_cpu[NAME][FAMILY],          _("Family"));
-	asprintf(&data->tab_cpu[NAME][EXTFAMILY],       _("Ext. Family"));
+	asprintf(&data->tab_cpu[NAME][EXTFAMILY],       _("Disp. Family"));
 	asprintf(&data->tab_cpu[NAME][MODEL],           _("Model"));
-	asprintf(&data->tab_cpu[NAME][EXTMODEL],        _("Ext. Model"));
+	asprintf(&data->tab_cpu[NAME][EXTMODEL],        _("Disp. Model"));
 	asprintf(&data->tab_cpu[NAME][TEMPERATURE],     _("Temp."));
 	asprintf(&data->tab_cpu[NAME][STEPPING],        _("Stepping"));
 	asprintf(&data->tab_cpu[NAME][INSTRUCTIONS],    _("Instructions"));
@@ -149,6 +149,7 @@ static void labels_setname(Labels *data)
 		asprintf(&data->tab_graphics[NAME][GPU1DRIVER      + i], _("Driver"));
 		asprintf(&data->tab_graphics[NAME][GPU1UMD         + i], _("UMD Version"));
 		asprintf(&data->tab_graphics[NAME][GPU1MODEL       + i], _("Model"));
+		asprintf(&data->tab_graphics[NAME][GPU1CU          + i], _("Compute Unit"));
 		asprintf(&data->tab_graphics[NAME][GPU1DIDRID      + i], _("DeviceID"));
 		asprintf(&data->tab_graphics[NAME][GPU1PCIE        + i], _("Interface"));
 		asprintf(&data->tab_graphics[NAME][GPU1TEMPERATURE + i], _("Temperature"));
@@ -158,6 +159,7 @@ static void labels_setname(Labels *data)
 		asprintf(&data->tab_graphics[NAME][GPU1CORECLOCK   + i], _("GPU clock"));
 		asprintf(&data->tab_graphics[NAME][GPU1MEMCLOCK    + i], _("Memory clock"));
 		asprintf(&data->tab_graphics[NAME][GPU1MEMUSED     + i], _("Memory Used"));
+		asprintf(&data->tab_graphics[NAME][GPU1REBAR       + i], _("Resizable BAR"));
 	}
 	asprintf(&data->objects[FRAMCARDS], _("Cards")); // Frame label
 
@@ -758,6 +760,7 @@ int main(int argc, char *argv[])
 	{
 		.gpu_driver  = { GPUDRV_UNKNOWN },
 		.device_path = { NULL },
+		.bar_size    = { 0 }
 	};
 	data->b_data = &(BenchData)
 	{
